import logging
from collections import Counter
from typing import Dict, List, Tuple

import spacy
from spacy.matcher import DependencyMatcher

logger = logging.getLogger(__name__)


def your_solution(animal: str, doc_list: List[Dict[str, str]]) -> List[Tuple[str, int]]:
    """
    Task: Extract things that the given animal eats. These things should be mentioned in the given list of documents.
    Each document in ``doc_list`` is a dictionary with keys ``animal``, ``url``, ``title`` and ``text``, whereas
    ``text`` points to the content of the document.

    :param animal: The animal to extract diets for.
    :param doc_list: A list of retrieved documents.
    :return: A list of things that the animal eats along with their frequencies.
    """

    #print(f"Animal: \"{animal}\". Number of documents: {len(doc_list)}.")

    # You can directly use the following list of documents, which is a list of str, if you don't need other information (i.e., url, title).
    documents = [doc["text"] for doc in doc_list]

    logger.info(
        f"Animal: \"{animal}\". Number of documents: {len(documents)}. Running SpaCy...")

    pattern_prep1 = [
        {
            "RIGHT_ID": "eat",
            "RIGHT_ATTRS": {"ORTH": "prey"}
        },
        {
            "LEFT_ID": "eat",
            "REL_OP": ">",
            "RIGHT_ID": "prep",
            "RIGHT_ATTRS": {"ORTH": {"IN": ["on", "upon"]}}
        },
        {
            "LEFT_ID": "prep",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj",
            "RIGHT_ATTRS": {"DEP": "pobj"}
        },
    ]

    pattern_prep2 = pattern_prep1+[
        {
            "LEFT_ID": "object_prep_obj",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj_conj1",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_prep3 = pattern_prep2 + [
        {
            "LEFT_ID": "object_prep_obj_conj1",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj_conj2",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_prep4 = pattern_prep3 + [
        {
            "LEFT_ID": "object_prep_obj_conj2",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj_conj3",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_prep5 = pattern_prep4+[
        {
            "LEFT_ID": "object_prep_obj_conj3",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj_conj4",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_hearst_prep1 = [
        {
            "RIGHT_ID": "eat",
            "RIGHT_ATTRS": {"ORTH": "prey"}
        },
        {
            "LEFT_ID": "eat",
            "REL_OP": ">",
            "RIGHT_ID": "prep",
            "RIGHT_ATTRS": {"ORTH": {"IN": ["on", "upon"]}}
        },
        {
            "LEFT_ID": "prep",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep",
            "RIGHT_ATTRS": {"DEP": "pobj"}
        },
        {
            "LEFT_ID": "object_prep",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep2",
            "RIGHT_ATTRS": {"ORTH": {"IN": ["like", "as"]}}
        },
        {
            "LEFT_ID": "object_prep2",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj",
            "RIGHT_ATTRS": {"DEP": "pobj"}
        },
    ]

    pattern_hearst_prep2 = pattern_hearst_prep1+[
        {
            "LEFT_ID": "object_prep_obj",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj_conj1",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_hearst_prep3 = pattern_hearst_prep2 + [
        {
            "LEFT_ID": "object_prep_obj_conj1",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj_conj2",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_hearst_prep4 = pattern_hearst_prep3 + [
        {
            "LEFT_ID": "object_prep_obj_conj2",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj_conj3",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_hearst_prep5 = pattern_hearst_prep4+[
        {
            "LEFT_ID": "object_prep_obj_conj3",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj_conj4",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_hearst1 = [
        {
            "RIGHT_ID": "eat",
            "RIGHT_ATTRS": {"LEMMA": {"IN": ["eat", "consume"]}}
        },
        {
            "LEFT_ID": "eat",
            "REL_OP": ">",
            "RIGHT_ID": "object",
            "RIGHT_ATTRS": {"DEP": "dobj"}
        },
        {
            "LEFT_ID": "object",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep",
            "RIGHT_ATTRS": {"ORTH": {"IN": ["like", "as"]}}
        },
        {
            "LEFT_ID": "object_prep",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj",
            "RIGHT_ATTRS": {"DEP": "pobj"}
        },
    ]

    pattern_hearst2 = pattern_hearst1+[
        {
            "LEFT_ID": "object_prep_obj",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj_conj1",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_hearst3 = pattern_hearst2 + [
        {
            "LEFT_ID": "object_prep_obj_conj1",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj_conj2",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_hearst4 = pattern_hearst3 + [
        {
            "LEFT_ID": "object_prep_obj_conj2",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj_conj3",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_hearst5 = pattern_hearst4+[
        {
            "LEFT_ID": "object_prep_obj_conj3",
            "REL_OP": ">",
            "RIGHT_ID": "object_prep_obj_conj4",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_hearst_include_1 = [
        {
            "RIGHT_ID": "initial",
            "RIGHT_ATTRS": {"ORTH": "diet"}
        },
        {
            "LEFT_ID": "initial",
            "REL_OP": "<",
            "RIGHT_ID": "include",
            "RIGHT_ATTRS": {"ORTH": "includes"}
        },
        {
            "LEFT_ID": "include",
            "REL_OP": ">",
            "RIGHT_ID": "object",
            "RIGHT_ATTRS": {"DEP": "dobj"}
        },
        {
            "LEFT_ID": "object",
            "REL_OP": ">",
            "RIGHT_ID": "object_conj1",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_hearst_include_2 = pattern_hearst_include_1 + [
        {
            "LEFT_ID": "object_conj1",
            "REL_OP": ">",
            "RIGHT_ID": "object_conj2",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_hearst_include_3 = pattern_hearst_include_2+[
        {
            "LEFT_ID": "object_conj2",
            "REL_OP": ">",
            "RIGHT_ID": "object_conj3",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_hearst_include_4 = pattern_hearst_include_3+[
        {
            "LEFT_ID": "object_conj3",
            "REL_OP": ">",
            "RIGHT_ID": "object_conj4",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern_hearst_include_5 = pattern_hearst_include_4+[
        {
            "LEFT_ID": "object_conj4",
            "REL_OP": ">",
            "RIGHT_ID": "object_conj5",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern1 = [
        {
            "RIGHT_ID": "eat",
            "RIGHT_ATTRS": {"LEMMA": {"IN": ["eat", "consume"]}}
        },
        {
            "LEFT_ID": "eat",
            "REL_OP": ">",
            "RIGHT_ID": "object",
            "RIGHT_ATTRS": {"DEP": "dobj"}
        },
    ]

    pattern2 = pattern1+[{
        "LEFT_ID": "object",
        "REL_OP": ">",
        "RIGHT_ID": "object_conj1",
        "RIGHT_ATTRS": {"DEP": "conj"}
    }]

    pattern3 = pattern2 + [
        {
            "LEFT_ID": "object_conj1",
            "REL_OP": ">",
            "RIGHT_ID": "object_conj2",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern4 = pattern3+[
        {
            "LEFT_ID": "object_conj2",
            "REL_OP": ">",
            "RIGHT_ID": "object_conj3",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern5 = pattern4+[
        {
            "LEFT_ID": "object_conj3",
            "REL_OP": ">",
            "RIGHT_ID": "object_conj4",
            "RIGHT_ATTRS": {"DEP": "conj"}
        },
    ]

    pattern = [
        pattern1, pattern2, pattern3, pattern4, pattern5,
        pattern_prep1, pattern_prep2, pattern_prep3, pattern_prep4, pattern_prep5,
        pattern_hearst_prep1, pattern_hearst_prep2, pattern_hearst_prep3, pattern_hearst_prep4, pattern_hearst_prep5,
        pattern_hearst1, pattern_hearst2, pattern_hearst3, pattern_hearst4, pattern_hearst5,
        pattern_hearst_include_1, pattern_hearst_include_2, pattern_hearst_include_3, pattern_hearst_include_4, pattern_hearst_include_5
    ]

    nlp = spacy.load("en_core_web_sm")

    matcher = DependencyMatcher(nlp.vocab)
    matcher.add("FOOD", pattern)
    # matches=[]
    food = []
    for document in documents:
        matches = []
        doc = nlp(document)
        matches = matcher(doc)
        length = 0
        longest_match = 0

        if matches != []:
            for i in range(len(matches)):
                match_id, token_ids = matches[i]
                if len(token_ids) > length:
                    longest_match = i

            match_id, token_ids = matches[longest_match]
            for i in range(len(token_ids)):
                # print(doc[token_ids[i]].lemma_)
                if(doc[token_ids[i]].text == "leaves"):
                    food.append(doc[token_ids[i]].text)
                else:
                    food.append(doc[token_ids[i]].lemma_)

    to_remove = ["prey", "on", "include", "as", "like", "eat", "diet", "consume", "they", "what",
                 "upon", "food", "anything", "that", "this", "kg", "pound", "lb", "it", "much", "lot", "specie"]
    food_list = [x for x in food if x not in to_remove]

    # print("Animal",animal)
    # print(Counter(food_list).most_common())
    return Counter(food_list).most_common()

    # return [("grass", 10), ("fish", 3)]
